@@ -505,6 +505,8 @@ gail_switch_page_watcher (GSignalInvocationHint *ihint,
 static gint
 gail_focus_idle_handler (gpointer data)
 {
+  GDK_THREADS_ENTER();
+
   focus_notify_handler = 0;
   /*
    * The widget which was to receive focus may have been removed
@@ -512,7 +514,10 @@ gail_focus_idle_handler (gpointer data)
   if (!next_focus_widget)
     {
       if (next_focus_widget != data)
-        return FALSE;
+	{
+	  GDK_THREADS_LEAVE ();
+	  return FALSE;
+	}
     }
   else
     {
@@ -523,6 +528,7 @@ gail_focus_idle_handler (gpointer data)
     
   gail_focus_notify (data);
 
+  GDK_THREADS_LEAVE ();
   return FALSE; 
 }
 
@@ -563,7 +569,12 @@ gail_focus_notify (GtkWidget *widget)
         atk_obj  = gail_get_accessible_for_widget (focus_widget, &transient);
       else
         atk_obj = NULL;
-      atk_focus_tracker_notify (atk_obj);
+      /*
+       * Do not report focus on redundant object
+       */
+      if (atk_obj && 
+	  (atk_object_get_role(atk_obj) != ATK_ROLE_REDUNDANT_OBJECT))
+	  atk_focus_tracker_notify (atk_obj);
       if (atk_obj && transient)
         g_object_unref (atk_obj);
       if (subsequent_focus_widget)
@@ -751,7 +762,11 @@ gail_focus_object_destroyed (gpointer data)
 static void
 gail_focus_tracker (AtkObject *focus_object)
 {
-  if (focus_object)
+  /*
+   * Do not report focus on redundant object
+   */
+  if (focus_object && 
+      (atk_object_get_role(focus_object) != ATK_ROLE_REDUNDANT_OBJECT))
     {
       AtkObject *old_focus_object;
 
